﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using Autodesk.Revit.DB;
using Autodesk.Revit.UI;
using Autodesk.Revit.UI.Events;
using Microsoft.Win32;

namespace BIM.IFC.Export.UI
{
    public class IFCCommandOverrideApplication : IExternalApplication
    {
        #region IExternalApplication Members

        /// <summary>
        /// Implementation of Shutdown for the external application.
        /// </summary>
        /// <param name="application">The Revit application.</param>
        /// <returns>The result (typically Succeeded).</returns>
        public Result OnShutdown(UIControlledApplication application)
        {
            // Clean up
            RemoveOverride(application);

            return Result.Succeeded;
        }

        /// <summary>
        /// Implementation of Startup for the external application.
        /// </summary>
        /// <param name="application">The Revit application.</param>
        /// <returns>The result (typically Succeeded).</returns>
        public Result OnStartup(UIControlledApplication application)
        {
            // Register execution override
            AddOverride(application);

            return Result.Succeeded;
        }
        #endregion

        public void AddOverride(UIControlledApplication application)
        {
            if (m_overrideToggle == null)
            {
                m_overrideToggle = new IFCOverrideToggle(this);
                m_overrideExternalEvent = ExternalEvent.Create(m_overrideToggle);
                application.ApplicationClosing += CleanupOnShutdown;
            }
            m_overrideToggle.EnableOverride = true;
            m_overrideExternalEvent.Raise();
        }

        public void RemoveOverride(UIControlledApplication application)
        {
            if (m_overrideToggle == null)
            {
                m_overrideToggle = new IFCOverrideToggle(this);
                m_overrideExternalEvent = ExternalEvent.Create(m_overrideToggle);
                application.ApplicationClosing += CleanupOnShutdown;
            }
            m_overrideToggle.EnableOverride = false;
            m_overrideExternalEvent.Raise(); 

        }

        public void CleanupOnShutdown(object sender, EventArgs eventArgs)
        {
            m_overrideExternalEvent.Dispose();
            m_overrideExternalEvent = null;
        }

        class IFCOverrideToggle : IExternalEventHandler
        {
            public void Execute(UIApplication application)
            {
                if (EnableOverride)
                {
                    RevitCommandId commandId = RevitCommandId.LookupCommandId("ID_EXPORT_IFC");
                    AddInCommandBinding ifcCommandBinding = application.CreateAddInCommandBinding(commandId);
                    ifcCommandBinding.Executed += m_commandApplication.OnIFCExport; 
                }
                else
                {
                    RevitCommandId commandId = RevitCommandId.LookupCommandId("ID_EXPORT_IFC");
                    application.RemoveAddInCommandBinding(commandId);
                }
            }

            public string GetName()
            {
                return "IFCOverrideToggle";
            }

            public bool EnableOverride { get; set; }

            private IFCCommandOverrideApplication m_commandApplication;

            public IFCOverrideToggle(IFCCommandOverrideApplication commandApplication)
            {
                EnableOverride = true;
                m_commandApplication = commandApplication;
            }
        }

        private IFCOverrideToggle m_overrideToggle = null;
        private ExternalEvent m_overrideExternalEvent = null;
        private String m_mruExportPath = null;
        private String m_mruConfiguration = null;
        
        /// <summary>
        /// Implementation of the command binding event for the IFC export command.
        /// </summary>
        /// <param name="sender">The event sender (Revit UIApplication).</param>
        /// <param name="args">The arguments (command binding).</param>
        public void OnIFCExport(object sender, CommandEventArgs args)
        {
            try
            {
                // Prepare basic objects
                UIApplication uiApp = sender as UIApplication;
                UIDocument uiDoc = uiApp.ActiveUIDocument;
                Document doc = uiDoc.Document;

                IFCExportConfigurationsMap configurationsMap = new IFCExportConfigurationsMap();
                configurationsMap.Add(IFCExportConfiguration.GetInSession());
                configurationsMap.AddBuiltInConfigurations();
                configurationsMap.AddSavedConfigurations(doc);

                String mruSelection = null;
                if (m_mruConfiguration != null && configurationsMap.HasName(m_mruConfiguration))
                    mruSelection =  m_mruConfiguration;

                IFCExport mainWindow = new IFCExport(configurationsMap, mruSelection);
                mainWindow.ShowDialog();

                // If user chose to continue
                if (mainWindow.Result == IFCExportResult.ExportAndSaveSettings)
                {
                    // change options
                    IFCExportConfiguration selectedConfig = mainWindow.GetSelectedConfiguration();

                    // Prepare the export options
                    IFCExportOptions exportOptions = new IFCExportOptions();
                    selectedConfig.UpdateOptions(exportOptions, uiDoc.ActiveView.Id);  

                    // prompt for the file name
                    SaveFileDialog fileDialog = new SaveFileDialog();
                    fileDialog.AddExtension = true;

                    String defaultDirectory = m_mruExportPath != null ? m_mruExportPath : null;
                    
                    if (defaultDirectory == null)
                    {
                        String revitFilePath = doc.PathName;
                        if (!String.IsNullOrEmpty(revitFilePath))
                        {
                            defaultDirectory = Path.GetDirectoryName(revitFilePath);
                        }
                    }

                    if (defaultDirectory == null)
                    {
                        defaultDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                    }

                    String defaultFileName = doc.Title;
                    if (String.IsNullOrEmpty(defaultFileName))
                    {
                        defaultFileName = "Project";
                    }
                    else
                    {
                        defaultFileName = Path.GetFileNameWithoutExtension(defaultFileName);
                    }
                    String defaultExtension = mainWindow.GetFileExtension();

                    fileDialog.FileName = defaultFileName;
                    fileDialog.DefaultExt = defaultExtension;
                    fileDialog.Filter = mainWindow.GetFileFilter();
                    fileDialog.InitialDirectory = defaultDirectory;
                    bool? fileDialogResult = fileDialog.ShowDialog();
                
                    // If user chose to continue
                    if (fileDialogResult.HasValue && fileDialogResult.Value)
                    {
                        // Prompt the user for the file location and path
                        String fullName = fileDialog.FileName;
                        String path = Path.GetDirectoryName(fullName);
                        String fileName = Path.GetFileName(fullName);

                        // IFC export requires an open transaction, although no changes should be made
                        Transaction transaction = new Transaction(doc, "Export IFC");   
                        transaction.Start();
                        FailureHandlingOptions failureOptions = transaction.GetFailureHandlingOptions();
                        failureOptions.SetClearAfterRollback(false);
                        transaction.SetFailureHandlingOptions(failureOptions);

                        bool result = doc.Export(path, fileName, exportOptions); // pass in the options here
                   
                        if (!result)
                        {
                            TaskDialog taskDialog = new TaskDialog("Error exporting IFC file");
                            taskDialog.MainInstruction = "The IFC export process encountered an error.";
                            taskDialog.MainIcon = TaskDialogIcon.TaskDialogIconWarning;
                            taskDialog.Show();
                        }

                        // Always roll back the transaction started earlier
                        transaction.RollBack();

                        // Remember last successful export location
                        m_mruExportPath = path;    
                    }      
                }
                if (mainWindow.Result == IFCExportResult.SaveSettings || mainWindow.Result == IFCExportResult.ExportAndSaveSettings)
                {
                    configurationsMap = mainWindow.GetModifiedConfigurations();
                    configurationsMap.UpdateSavedConfigurations(doc);

                    // Remember last selected configuration
                    m_mruConfiguration = mainWindow.GetSelectedConfiguration().Name;
                }
            }
            catch (Exception e)
            {
                TaskDialog taskDialog = new TaskDialog("Error exporting IFC file");
                taskDialog.MainInstruction = "The IFC export process encountered an error.";
                taskDialog.MainIcon = TaskDialogIcon.TaskDialogIconWarning;
                taskDialog.ExpandedContent = e.ToString();
                taskDialog.Show();
            }
        }
    }
}
